/******************************************************************************
 * BackgammonBoard.java - Be a backgammon board
 * $Id: BackgammonBoard.java,v 1.6 2011/06/03 19:26:10 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: BackgammonBoard.java,v $
 * Revision 1.6  2011/06/03 19:26:10  dick
 * Add some debug opening boards.
 *
 * Revision 1.5  2011/06/03 17:05:48  dick
 * Define X=0 and O=1 so we can get playerScore[X] and playerScore[O].
 *
 * Revision 1.4  2011/06/03 04:14:07  dick
 * Attributes player[12]Got becomes player[12]Score.
 *
 * Revision 1.3  2011/06/01 20:26:18  dick
 * Deadwood delete.  Privatize privates.  Javadoc.
 *
 * Revision 1.2  2011/05/30 05:29:21  dick
 * retab.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.backgammon;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsSet;
import net.sourceforge.jibs.util.JibsWriter;

/**
 * This is the encapsulation of a backgammon board as needed for <b>Jibs</b>.<br>
 * Therefore it also models a match between two players of several games.
 * Each game uses a <i>real</i> Backgammon board to represent such a
 * particular game.<br>
 * One of the players is playing the X-pieces (represemted by negative
 * numbers) whereas the other is playing the O-pieces (represented by
 * positive numbers).<br>
 * <br>
 * This class uses an array of 26 ints to represemt the board internally.
 * 
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.6 $ <br> $Date: 2011/06/03 19:26:10 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/backgammon/BackgammonBoard.java">cvs BackgammonBoard.java</a>
 */
public class BackgammonBoard {
	/** Index into the X player */
	public static final int X = 0;
	/** Index into the O player */
	public static final int O = 1;

	//	private static Logger logger = Logger.getLogger(BackgammonBoard.class);
	char[] value = new char[] {13,10};
	public String newline = new String(value);
	private JibsServer jibsServer;
	private Player playerX;
	private Player playerO;
	private Die pDie1;
	private Die pDie2;
	private Die oDie1;
	private Die oDie2;
	private int bar;
	private int[] board;
	private int canMove;
	private int color;
	private int cubeNumber;
	private int didCrawford;
	private int direction;
	private int forcedMove;
	private int home;
	private int matchlength;
	private int mayDouble1;
	private int mayDouble2;
	private int onBar1;
	private int onBar2;
	private int onHome1;
	private int onHome2;
//	private int player1Score;
	private String player1Name;
//	private int player2Score;
	int[] playerScore = new int[2];
	private String player2Name;
	private int redoubles;
	private int turn;
	private int wasDoubled;
	private JibsMatch jibsMatchVersion;
	private Player joinPlayer1;
	private Player joinPlayer2;
	private boolean crawFordGame;
	private boolean isEnded;

	/** Construct a clone of the passed board
	 * @param b2 The board to make a copy of.
	 */
	public BackgammonBoard(final BackgammonBoard b2) {
		this.init(b2.jibsServer, b2.player1Name, b2.player2Name,
				b2.matchlength, b2.getMatchVersion());
		this.jibsServer = b2.jibsServer;
		this.playerX = b2.playerX;
		this.playerO = b2.playerO;
		this.pDie1.setJibsServer(b2.pDie1.getJibsServer());
		this.pDie2.setJibsServer(b2.pDie2.getJibsServer());
		this.oDie1.setJibsServer(b2.oDie1.getJibsServer());
		this.oDie2.setJibsServer(b2.oDie2.getJibsServer());
		this.pDie1.setValue(b2.pDie1.getValue());
		this.pDie2.setValue(b2.pDie2.getValue());
		this.oDie1.setValue(b2.oDie1.getValue());
		this.oDie2.setValue(b2.oDie2.getValue());
		this.bar = b2.bar;
		this.canMove = b2.canMove;
		this.color = b2.color;
		this.cubeNumber = b2.cubeNumber;
		this.didCrawford = b2.didCrawford;
		this.direction = b2.direction;
		this.forcedMove = b2.forcedMove;
		this.home = b2.home;
		this.matchlength = b2.matchlength;
		this.mayDouble1 = b2.mayDouble1;
		this.mayDouble2 = b2.mayDouble2;
		this.onBar1 = b2.onBar1;
		this.onBar2 = b2.onBar2;
		this.onHome1 = b2.onHome1;
		this.onHome2 = b2.onHome2;
		this.playerScore[O] = b2.playerScore[O];
		this.player1Name = b2.player1Name;
		this.playerScore[X] = b2.playerScore[X];
		this.player2Name = b2.player2Name;
		this.redoubles = b2.redoubles;
		this.turn = b2.turn;
		this.wasDoubled = b2.wasDoubled;
		this.jibsMatchVersion = b2.jibsMatchVersion;
		this.joinPlayer1 = b2.joinPlayer1;
		this.joinPlayer2 = b2.joinPlayer2;
		this.crawFordGame = b2.crawFordGame;

		for (int i = 0; i < 26; i++) {
			this.board[i] = b2.board[i];
		}

		this.isEnded = b2.isEnded;
	}

	/** Construct a board with these values.
	 * @param jibsServer
	 * @param player1Name
	 * @param player2Name
	 * @param matchlength
	 * @param matchVersion
	 */
	public BackgammonBoard(JibsServer jibsServer, String player1Name,
			String player2Name, int matchlength,
			JibsMatch matchVersion) {
		this.init(jibsServer, player1Name, player2Name, matchlength,
				matchVersion);
	}

	/**
	 *
	 */
	private void init(JibsServer jibsServer, String player1Name,
			String player2Name, int matchlength, JibsMatch matchVersion) {
		this.player1Name = player1Name;
		this.player2Name = player2Name;
		this.matchlength = matchlength;
		this.player1Name = player1Name;
		this.player2Name = player2Name;
		this.jibsMatchVersion = matchVersion;

		pDie1 = new Die(jibsServer);
		pDie2 = new Die(jibsServer);
		oDie1 = new Die(jibsServer);
		oDie2 = new Die(jibsServer);

		cubeNumber = 1;
		playerScore[O] = 0;
		playerScore[X] = 0;
		board = new int[26];

		// standard backgammon board
		for (int i = 0; i < board.length; i++) {
			board[i] = 0;
		}

		// Standard opening
		board[24] = 2;
		board[13] = 5;
		board[8] = 3;
		board[6] = 5;
		board[1] = -2;
		board[12] = -5;
		board[17] = -3;
		board[19] = -5;

/*
		board[1]=-2; board[3] = 2; board[4] = 2; board[6] = 3; board[8] = 1;
		board[12] = -4; board[13] = 5; board[17] = -4; board[19] = -5;
		board[24] = 2; //board[0]=2; // Bar von 'X' //board[25]=2; // Bar von 'O'
*/
/*		// Debug for a gammon
		board[1] = 4;
		board[4] = -5;
		board[5] = -5;
		board[6] = -5;
*/
/*		// Debug for a gammon the other way
		board[24] = -4;
		board[23] = 5;
		board[22] = 5;
		board[21] = 5;
*/
		forcedMove = 0;
		didCrawford = 0;
		redoubles = 0;
		mayDouble1 = 1;
		mayDouble2 = 1;
		wasDoubled = 0;
		color = -1;
		direction = 1;
		home = 25;
		bar = 0;

		// board[0]=2; // Bar 'X'
		int totalPlayer1 = board[0];
		int totalPlayer2 = board[25];

		for (int i = 1; i <= 24; i++) {
			if (board[i] > 0) {
				totalPlayer2 += Math.abs(board[i]);
			}

			if (board[i] < 0) {
				totalPlayer1 += Math.abs(board[i]);
			}
		}

		onHome1 = 15 - totalPlayer1;
		onHome2 = 15 - totalPlayer2;

		onBar1 = board[0];
		onBar2 = board[25];
		this.isEnded = false;
	}

	/**
	 *
	 * @return
	 */
	public int getBar() {
		return bar;
	}

	/**
	 *
	 * @return
	 */
	public int[] getBoard() {
		return board;
	}

	/**
	 *
	 * @return
	 */
	public int getCanMove() {
		return canMove;
	}

	/**
	 *
	 * @return
	 */
	public int getColor() {
		return color;
	}

	/**
	 *
	 * @return
	 */
	public int getCubeNumber() {
		return cubeNumber;
	}

	/**
	 *
	 * @return
	 */
	public int getDidCrawford() {
		return didCrawford;
	}

	/**
	 *
	 * @return
	 */
	public int getDirection() {
		return direction;
	}

	/**
	 *
	 */
	public int getForcedMove() {
		return forcedMove;
	}

	/**
	 *
	 * @return
	 */
	public int getHome() {
		return home;
	}

	/**
	 *
	 * @return
	 */
	public int getMatchlength() {
		return matchlength;
	}

	/**
	 *
	 * @return
	 */
	public int getMayDouble1() {
		return mayDouble1;
	}

	/**
	 *
	 * @return
	 */
	public int getMayDouble2() {
		return mayDouble2;
	}

	/**
	 *
	 * @return
	 */
	public int getOnBar1() {
		return onBar1;
	}

	/**
	 *
	 * @return
	 */
	public int getOnBar2() {
		return onBar2;
	}

	/**
	 *
	 * @return
	 */
	public int getOnHome1() {
		return onHome1;
	}

	/**
	 *
	 * @return
	 */
	public int getOnHome2() {
		return onHome2;
	}

	public int getOnHome(int XO) {
		if (XO == X)
			return(onHome1);
		return(onHome2);
	}
	/**
	 *
	 * @param game
	 * @return
	 */
	public String getOutputBoard(JibsGame game) {
		StringBuffer strBoard = new StringBuffer();

		strBoard.append("board:");
		strBoard.append("You:");
		strBoard.append(player2Name);
		strBoard.append(":");
		strBoard.append(matchlength);
		strBoard.append(":");
		strBoard.append(playerScore[X]);
		strBoard.append(":");
		strBoard.append(playerScore[O]);

		for (int i = 0; i < 26; i++) {
			strBoard.append(":");
			strBoard.append(board[i]);
		}

		strBoard.append(":");
		strBoard.append(turn);
		strBoard.append(":");
		strBoard.append(getpDie1().getValue());
		strBoard.append(":");
		strBoard.append(getpDie2().getValue());
		strBoard.append(":");
		strBoard.append(getoDie1().getValue());
		strBoard.append(":");
		strBoard.append(getoDie2().getValue());
		strBoard.append(":");
		strBoard.append(cubeNumber);
		strBoard.append(":");
		strBoard.append(mayDouble1);
		strBoard.append(":");
		strBoard.append(mayDouble2);
		strBoard.append(":");
		strBoard.append(wasDoubled);
		strBoard.append(":");
		strBoard.append(color);
		strBoard.append(":");
		strBoard.append(direction);
		strBoard.append(":");
		strBoard.append(home);
		strBoard.append(":");
		strBoard.append(bar);
		strBoard.append(":");
		strBoard.append(onHome1);
		strBoard.append(":");
		strBoard.append(onHome2);
		strBoard.append(":");
		strBoard.append(onBar1);
		strBoard.append(":");
		strBoard.append(onBar2);
		strBoard.append(":");
		strBoard.append(canMove);
		strBoard.append(":");
		strBoard.append(forcedMove);
		strBoard.append(":");
		strBoard.append(didCrawford);
		strBoard.append(":");
		strBoard.append(redoubles);

		return strBoard.toString();
	}

	/**
	 *
	 * @return
	 */
	public int getPlayer1Score() {
		return playerScore[X];
	}

	/**
	 *
	 * @return
	 */
	public String getPlayer1Name() {
		return player1Name;
	}

	/**
	 *
	 * @return
	 */
	public int getPlayer2Score() {
		return playerScore[O];
	}

	/**
	 *
	 * @return
	 */
	public String getPlayer2Name() {
		return player2Name;
	}

	/**
	 *
	 * @return
	 */
	public int getRedoubles() {
		return redoubles;
	}

	/**
	 *
	 * @return
	 */
	public int getTurn() {
		return turn;
	}

	/**
	 *
	 * @return
	 */
	public int getWasDoubled() {
		return wasDoubled;
	}

	/**
	 *
	 * @param out1
	 * @param i
	 * @param dice1
	 * @param dice2
	 * @param j
	 * @param k
	 */
	public void outBoard(JibsWriter out1, String name, int i, int dice1,
			int dice2, int j, int k) {
		Player player = getPlayerX();
		JibsSet jibsSet = player.getJibsSet();
		String strBoardStyle = (String) jibsSet.get("boardstyle");
		int boardstyle = new Integer(strBoardStyle).intValue();

		switch (boardstyle) {
		case 1:default:
		case 2:
			outBoard_2(out1, i, dice1, dice2, j, k);

			break;

		case 3:

			String x = outBoard_3(out1, name, i, dice1, dice2, j, k);
			out1.println(x);

			break;
		}
	}

	/**
	 *
	 * @param out1
	 * @param turn
	 * @param dice11
	 * @param dice12
	 * @param dice21
	 * @param dice22
	 */
	public void outBoard_2(JibsWriter out1, int turn, int dice11, int dice12,
			int dice21, int dice22) {
		int lines;
		int i;
		StringBuffer bf = new StringBuffer();

		bf.append(newline);
		bf.append("+-1--2--3--4--5--6-------7--8--9-10-11-12-+");
		bf.append(" o:");
		bf.append(player2Name);
		bf.append(" - score: ");
		bf.append(playerScore[O]);

		for (lines = 1; lines <= 5; lines++) {
			bf.append(newline);
			bf.append("|");

			for (i = 1; i <= 6; i++) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|   |");

			for (i = 7; i <= 12; i++) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|");
		}

		bf.append(newline);
		bf.append("|                  |BAR|                  | v ");

		if (getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			bf.append(matchlength);
			bf.append("-point match");
		} else {
			bf.append("unlimited match");
		}

		for (lines = 5; lines >= 1; lines--) {
			bf.append("\r\n|");

			for (i = 24; i >= 19; i--) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|   |");

			for (i = 18; i >= 13; i--) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|");
		}

		bf.append(newline);
		bf.append("+24-23-22-21-20-19------18-17-16-15-14-13-+");
		bf.append(" x:");
		bf.append(player1Name);
		bf.append(" - score: ");
		bf.append(playerScore[X]);
		bf.append(newline);
		bf.append("BAR: O:");
		bf.append(onBar2);
		bf.append(" X:");
		bf.append(onBar1);
		bf.append("   OFF: O:");
		bf.append(onHome2);
		bf.append(" X:");
		bf.append(onHome1);
		bf.append("   Cube: ");
		bf.append(cubeNumber);

		if (dice11 > 0) {
			bf.append("   You rolled " + dice11 + " and " + dice12 + ".");
		}

		out1.println(bf.toString());
	}

	/**
	 *
	 * @param out1
	 * @param turn
	 * @param dice11
	 * @param dice12
	 * @param dice21
	 * @param dice22
	 * @return
	 */
	public String outBoard_3(JibsWriter out1, String name, int turn,
			int dice11, int dice12, int dice21, int dice22) {
		StringBuffer strBoard = new StringBuffer();

		strBoard.append("board:");
		strBoard.append(name);
		strBoard.append(":");
		strBoard.append(getPlayer2Name());
		strBoard.append(":");

		if (getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			strBoard.append(matchlength);
		} else {
			strBoard.append("9999");
		}

		strBoard.append(":");
		strBoard.append(playerScore[X]);
		strBoard.append(":");
		strBoard.append(playerScore[O]);

		for (int i = 0; i < 26; i++) {
			strBoard.append(":");
			strBoard.append(board[i]);
		}

		strBoard.append(":");
		strBoard.append(turn);
		strBoard.append(":");
		strBoard.append(dice11);
		strBoard.append(":");
		strBoard.append(dice12);
		strBoard.append(":");
		strBoard.append(dice21);
		strBoard.append(":");
		strBoard.append(dice22);
		strBoard.append(":");
		strBoard.append(cubeNumber);
		strBoard.append(":");
		strBoard.append(mayDouble1);
		strBoard.append(":");
		strBoard.append(mayDouble2);
		strBoard.append(":");
		strBoard.append(wasDoubled);
		strBoard.append(":");
		strBoard.append(color);
		strBoard.append(":");
		strBoard.append(direction);
		strBoard.append(":");
		strBoard.append(home);
		strBoard.append(":");
		strBoard.append(bar);
		strBoard.append(":");
		strBoard.append(onHome1);
		strBoard.append(":");
		strBoard.append(onHome2);
		strBoard.append(":");
		strBoard.append(onBar1);
		strBoard.append(":");
		strBoard.append(onBar2);
		strBoard.append(":");
		strBoard.append(canMove);
		strBoard.append(":");
		strBoard.append(forcedMove);
		strBoard.append(":");
		strBoard.append(didCrawford);
		strBoard.append(":");
		strBoard.append(redoubles);

		return strBoard.toString();
	}

	/**
	 *
	 * @param mv
	 * @return
	 */
	public BackgammonBoard placeMoveO(Move mv) {
		BackgammonBoard helpBoard = new BackgammonBoard(this);
		int source = mv.source;
		int destination = mv.destination;

		if (source == 25) { // moving from bar
			helpBoard.board[0]--;
			helpBoard.onBar2--;
		} else {
			helpBoard.board[source]--;
		}

		if (destination <= 0) {
			// bearing off
			helpBoard.onHome2++;
		} else {
			helpBoard.board[destination]++;

			if (helpBoard.board[destination] == 0) {
				helpBoard.board[destination]++;
				helpBoard.board[0]++;
				helpBoard.onBar1++;
			}

			if (destination <= 0) {
				helpBoard.onHome2++;
			}
		}

		return helpBoard;
	}

	/**
	 * Perform a move on this board.
	 *
	 * @param mv
	 *            a half-move
	 *
	 * @return A modified backgammon board where the move has been placed on.
	 */
	public BackgammonBoard placeMoveX(Move mv) {
		BackgammonBoard helpBoard = new BackgammonBoard(this);
		int source = mv.source;
		int destination = mv.destination;

		if (source == bar) { // moving from bar	// XXX: This seems wrong.  Using bar as a const?? maybe?
			helpBoard.board[0]--;
			helpBoard.onBar1--;
		} else if (source >= 1 && source <= 24) {
			helpBoard.board[source]++;
		}

		if (destination <= 0) {
			// bearing off
			helpBoard.onHome1++;
		} else {
			if ((destination >= 1) && (destination <= 24)) {
				helpBoard.board[destination]--;

				if (helpBoard.board[destination] == 0) {
					helpBoard.board[destination]--;
					helpBoard.board[25]++;
					helpBoard.onBar2++;
				}
			}

			if (destination >= 25) {
				helpBoard.onHome1++;
			}
		}

		return helpBoard;
	}

	/**
	 *
	 * @param bar
	 */
	public void setBar(int bar) {
		this.bar = bar;
	}

	/**
	 *
	 * @param board
	 */
	public void setBoard(int[] board) {
		this.board = board;
	}

	/**
	 *
	 * @param canMove
	 */
	public void setCanMove(int canMove) {
		this.canMove = canMove;
	}

	/**
	 *
	 * @param color
	 */
	public void setColor(int color) {
		this.color = color;
	}

	/**
	 *
	 * @param cubeNumber
	 */
	public void setCubeNumber(int cubeNumber) {
		this.cubeNumber = cubeNumber;
	}

	/**
	 *
	 * @param didCrawford
	 */
	public void setDidCrawford(int didCrawford) {
		this.didCrawford = didCrawford;
	}

	/**
	 *
	 * @param direction
	 */
	public void setDirection(int direction) {
		this.direction = direction;
	}

	/**
	 *
	 * @param forcedMove
	 */
	public void setForcedMove(int forcedMove) {
		this.forcedMove = forcedMove;
	}

	/**
	 *
	 * @param home
	 */
	public void setHome(int home) {
		this.home = home;
	}

	/**
	 *
	 * @param matchlength
	 */
	public void setMatchlength(int matchlength) {
		this.matchlength = matchlength;
	}

	/**
	 *
	 * @param mayDouble1
	 */
	public void setMayDouble1(int mayDouble1) {
		this.mayDouble1 = mayDouble1;
	}

	/**
	 *
	 * @param mayDouble2
	 */
	public void setMayDouble2(int mayDouble2) {
		this.mayDouble2 = mayDouble2;
	}

	/**
	 *
	 * @param onBar1
	 */
	public void setOnBar1(int onBar1) {
		this.onBar1 = onBar1;
	}

	/**
	 *
	 * @param onBar2
	 */
	public void setOnBar2(int onBar2) {
		this.onBar2 = onBar2;
	}

	/**
	 *
	 * @param onHome1
	 */
	public void setOnHome1(int onHome1) {
		this.onHome1 = onHome1;
	}

	/**
	 *
	 * @param onHome2
	 */
	public void setOnHome2(int onHome2) {
		this.onHome2 = onHome2;
	}

	/**
	 *
	 * @param player1Score
	 */
	public void setPlayerScore(int XO, int playerScore) {
		this.playerScore[XO] = playerScore;
	}

	/** Add these points to player1's score.
	 * @param XO These points go to either player X or player O
	 * @param points The number of points he won this game.
	 */
	public void addPlayerScore(int XO, int points) {
		this.playerScore[XO] += points;
	}

	/** Get one of the players scores
	 * @param XO Do we want player X or player O's score
	 * @return Player X's or Player O's score
	 */
	public int getPlayerScore(int XO) {
		return(this.playerScore[XO]);
	}

	/**
	 *
	 * @param score
	 */
	public void setPlayerXScore(int score) {
		playerScore[X] = score;
	}

	/**
	 *
	 * @param score
	 */
	public void setPlayerOScore(int score) {
		playerScore[O] = score;
	}


	/**
	 *
	 * @param player1Name
	 */
	public void setPlayer1Name(String player1Name) {
		this.player1Name = player1Name;
	}

	/**
	 *
	 * @param player2Score
	 */
//	public void setPlayer2Score(int player2Score) {
//		this.player2Score = player2Score;
//	}

	/** Add these points to player2's score.
	 * @param points The number of points he won this game.
	 */
//	public void addPlayer2Score(int points) {
//		this.player2Score += points;
//	}

	/**
	 *
	 * @param player2Name
	 */
	public void setPlayer2Name(String player2Name) {
		this.player2Name = player2Name;
	}

	/**
	 *
	 * @param redoubles
	 */
	public void setRedoubles(int redoubles) {
		this.redoubles = redoubles;
	}

	/**
	 *
	 * @param turn
	 */
	public void setTurn(int turn) {
		this.turn = turn;
	}

	/**
	 *
	 * @param wasDoubled
	 */
	public void setWasDoubled(int wasDoubled) {
		this.wasDoubled = wasDoubled;
	}

	/**
	 *
	 * @return
	 */
	public Player getPlayerX() {
		return playerX;
	}

	/**
	 *
	 * @return
	 */
	public Player getPlayerO() {
		return playerO;
	}

	/**
	 *
	 * @param matchVersion
	 */
	public void setMatchVersion(JibsMatch matchVersion) {
		this.jibsMatchVersion = matchVersion;
	}

	/**
	 *
	 * @param d1
	 */
	public void setpDie1(Die d1) {
		pDie1.setValue(d1.getValue());
	}

	/**
	 *
	 * @return
	 */
	public Die getpDie1() {
		return pDie1;
	}

	/**
	 *
	 * @param d2
	 */
	public void setpDie2(Die d2) {
		pDie2.setValue(d2.getValue());
	}

	/**
	 *
	 * @return
	 */
	public Die getpDie2() {
		return pDie2;
	}

	/**
	 *
	 * @param playerX
	 */
	public void setPlayerX(Player playerX) {
		this.playerX = playerX;
	}

	/**
	 *
	 * @param playerO
	 */
	public void setPlayerO(Player playerO) {
		this.playerO = playerO;
	}

	/**
	 *
	 * @param die
	 */
	public void setoDie1(Die die) {
		oDie1.setValue(die.getValue());
	}

	/**
	 *
	 * @return
	 */
	public Die getoDie1() {
		return oDie1;
	}

	/**
	 *
	 * @return
	 */
	public Die getoDie2() {
		return oDie2;
	}

	/**
	 *
	 * @param die
	 */
	public void setoDie2(Die die) {
		oDie2.setValue(die.getValue());
	}

	/**
	 *
	 * @return
	 */
	public JibsMatch getMatchVersion() {
		return jibsMatchVersion;
	}

	/**
	 *
	 * @param player
	 * @return
	 */
	public Player getOpponent(Player player) {
		if (player.getName().equals(playerX.getName())) {
			return playerO;
		}

		if (player.getName().equals(playerO.getName())) {
			return playerX;
		}

		return null;
	}

	/**
	 *
	 * @return
	 */
	public int getPlayerXScore() {
		return playerScore[X];
	}

	/**
	 *
	 * @return
	 */
	public int getPlayerOScore() {
		return playerScore[O];
	}

	/**
	 *
	 * @param player
	 * @return
	 */
	public boolean isPlayerX(Player player) {
		if (player.getName().equals(this.playerX.getName())) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 *
	 * @return
	 */
	public boolean isEnded() {
		return isEnded;
	}

	/**
	 *
	 * @param player
	 */
	public void setJoinPlayer(Player player) {
		if (joinPlayer1 == null) {
			joinPlayer1 = player;
		} else {
			joinPlayer2 = player;
		}
	}

	/**
	 *
	 * @return
	 */
	public Object getJoinPlayer1() {
		return joinPlayer1;
	}

	/**
	 *
	 * @return
	 */
	public Object getJoinPlayer2() {
		return joinPlayer2;
	}

	/**
	 *
	 * @param b
	 */
	public void setPostCrawFord(boolean b) {
		// TODO Auto-generated method stub
	}

	/**
	 *
	 * @return
	 */
	public boolean isCrawFordGame() {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 *
	 * @param b
	 */
	public void setCrawFordGame(boolean b) {
		crawFordGame = b;
	}

	/**
	 *
	 * @param object
	 */
	public void setJoinPlayer1(Player object) {
		joinPlayer1 = object;
	}

	/**
	 *
	 * @param object
	 */
	public void setJoinPlayer2(Player object) {
		joinPlayer2 = object;
	}

	/** Create a copy of this board from the opponent's point of view.
	 * @return The mirror of this board
	 */
	public BackgammonBoard switch2O() {
		BackgammonBoard opBoard = new BackgammonBoard(this);
		opBoard.setPlayer1Name(getPlayer2Name());
		opBoard.setPlayer2Name(getPlayer1Name());
		opBoard.setDirection(-1);
		opBoard.setColor(1);
		opBoard.getpDie1().setValue(getoDie1().getValue());
		opBoard.getpDie2().setValue(getoDie2().getValue());
		opBoard.getoDie1().setValue(getpDie1().getValue());
		opBoard.getoDie2().setValue(getpDie2().getValue());
		opBoard.setOnBar1(getOnBar2());
		opBoard.setOnBar2(getOnBar1());
		opBoard.setOnHome1(getOnHome2());
		opBoard.setOnHome2(getOnHome1());
		opBoard.setBar(getHome());
		opBoard.setHome(getBar());
		opBoard.setPlayerXScore(getPlayerXScore());
		opBoard.setPlayerOScore(getPlayerOScore());
		opBoard.setMayDouble1(getMayDouble2());
		opBoard.setMayDouble2(getMayDouble1());

		return opBoard;
	}

	public void setEnded(boolean isEnded) {
		this.isEnded = isEnded;
	}
}
